@@ -8,7 +8,7 @@ use IO::Socket;
 use Time::HiRes qw(time);
 use POSIX ":sys_wait_h"; # WNOHANG
 
-our $VERSION = 1.16;
+our $VERSION = 1.20;
 
 sub new {
 	my ($class, %args) = @_;
@@ -26,7 +26,7 @@ sub child_stop {
 	my ($rc, $sig, $core) = ($status >> 8, $status & 127, $status & 128);
 	my $stoptime = sprintf "%.02f", time() - $child->{start};
 	my %runtime = (exists $child->{runtime}) ? (runtime => $child->{runtime}) : ();
-	my %id = (exists $child->{id}) ? (id => exists $child->{id}) : ();
+	my %id = (exists $child->{id}) ? (id => $child->{id}) : ();
 	my %reason = ();
 
 	if ($^O eq 'MSWin32') { # cpantester: strawberry perl does not support WIF calls
@@ -65,8 +65,9 @@ sub childs_reap {
 			# chance to cleanup resources, and send nonmaskable
 			# sigkill to terminate child only after that.
 			# (note: some nfs-syscalls might block nevertheless)
+			# (note: sigkill on windows might deadlock - see below)
 			#
-			if (($waittime > 2) && ($child->{state} ne 'kill')) {
+			if (($waittime > 2) && ($child->{state} ne 'kill') && ($^O ne 'MSWin32')) {
 				warn "worker child $child->{pid}: still running after $child->{state} - send kill\n";
 				kill 'KILL', $child->{pid};
 				$child->{state} = 'kill';
@@ -76,23 +77,57 @@ sub childs_reap {
 				$child->{state} = 'term';
 			}
 		} elsif ($res < 0) {
+			return 0 if (($flags == 0) && $!{EINTR}); # blocking waitpid might be interrupted by signal
 			warn "worker child $child->{pid}: disappeared" unless ($^O eq 'MSWin32');
 			$self->child_stop($child->{pid}, 0);
 		} else {
 			$self->child_stop($child->{pid}, $?);
 		}
 	}
+	return 1;
+}
+
+# For perl on windows the perlfork emulation for threading systems
+# will be used internally (see: https://perldoc.perl.org/perlfork).
+#
+# The description of kill() notes that using sig-KILL the process
+# which implements the pseudo-processes can be blocked and the perl
+# interpreter hangs.
+#
+# -> this behavior is seen even on current win10/2023 systems with
+#    an interpreter like strawberry-perl-5.32.1-64bit (cpan testers).
+#
+# This should not happen for sig-TERM, where the perlfork-emulation
+# will not try to wait on childs and leak resources to avoid deadlocks.
+#
+# So the Async module should be avoided on windows, because it will
+# not work as desired.
+#
+sub _childs_term {
+	my ($self) = @_;
+
+	foreach my $child (values %{$self->{finished}}) {
+		if ($child->{state} ne 'term') {
+			warn "worker child $child->{pid}: still running after $child->{state} - force term\n";
+			kill 'TERM', $child->{pid};
+			$child->{state} = 'term';
+		}
+	}
+	return;
 }
 
 sub childs_kill {
 	my ($self) = @_;
 
+	return $self->_childs_term() if ($^O eq 'MSWin32');
+
 	foreach my $child (values %{$self->{finished}}) {
 		if ($child->{state} ne 'kill') {
 			kill 'KILL', $child->{pid}; # terminate non maskable
 			$child->{state} = 'kill';
 		}
 	}
+	return;
 }
 
 sub job_add {
@@ -102,6 +137,7 @@ sub job_add {
 	$child->{start} = time();
 	@$child{keys %$meta} = values %$meta;
 	$self->{jobs}{$child->{reader}->fileno} = $child;
+	return;
 }
 
 sub job_rem {
@@ -110,6 +146,7 @@ sub job_rem {
 	$child->{runtime} = sprintf "%.02f", time() - $child->{start};
 	$child->{start} = time();
 	delete $self->{jobs}{$child->{reader}->fileno};
+	return;
 }
 
 sub child_finish {
@@ -129,8 +166,9 @@ sub child_finish {
 }
 
 sub child_start {
-	my ($self, $worker, $msg_id) = @_;
+	my ($self, $worker, $msg_id, $msg_vci) = @_;
 	my %id = (defined $msg_id) ? (id => $msg_id) : ();
+	my %vci = (defined $msg_vci) ? (vci => $msg_vci) : ();
 
 	# Handle waitpid() explicitly instead of using open('-|');
 	# see: https://perldoc.perl.org/perlipc#Safe-Pipe-Opens
@@ -142,7 +180,7 @@ sub child_start {
 
 	if ($pid != 0) { # parent
 		my $child = {reader => $rd, pid => $pid, start => time(), %id};
-		$self->{trace_cb}->('RUN', {pid => $pid, %id}) if $self->{trace_cb};
+		$self->{trace_cb}->('RUN', {pid => $pid, %id, %vci}) if $self->{trace_cb};
 		close($wr);
 		return $child;
 	}
@@ -163,6 +201,7 @@ sub msg_enqueue {
 
 	# queue message and start child from rpc_handler
 	push(@{$self->{jobqueue}}, $msg);
+	return;
 }
 
 sub msg_dequeue {
